@@ -1,13 +1,13 @@
 package Locale::Places::DB;
 
-=head1
+=head1 NAME
 
 Locale::Places::DB
 
 =cut
 
 # Author Nigel Horne: njh@bandsman.co.uk
-# Copyright (C) 2015-2022, Nigel Horne
+# Copyright (C) 2015-2023, Nigel Horne
 
 # Usage is subject to licence terms.
 # The licence terms of this software are as follows:
@@ -65,6 +65,42 @@ use Carp;
 our $directory;
 our $logger;
 our $cache;
+our $cache_duration;
+
+=head1 SUBROUTINES/METHODS
+
+=head2 init
+
+Set some class level defaults.
+
+    __PACKAGE__::DB::init(directory => '../databases')
+
+See the documentation for new() to see what variables can be set
+
+=cut
+
+sub init {
+	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+
+	$directory ||= $args{'directory'};
+	$logger ||= $args{'logger'};
+	$cache ||= $args{'cache'};
+	$cache_duration ||= $args{'cache_duration'};
+}
+
+=head2 new
+
+Create an object to point to a read-only database.
+
+Arguments:
+
+cache => place to store results
+cache_duration => how long to store results in the cache (default is 1 hour)
+directory => where the database file is held
+
+If the arguments are not set, tries to take from class level defaults
+
+=cut
 
 sub new {
 	my $proto = shift;
@@ -83,38 +119,31 @@ sub new {
 		logger => $args{'logger'} || $logger,
 		directory => $args{'directory'} || $directory,	# The directory containing the tables in XML, SQLite or CSV format
 		cache => $args{'cache'} || $cache,
+		cache_duration => $args{'cache_duration'} || $cache_duration || '1 hour',
 		table => $args{'table'},	# The name of the file containing the table, defaults to the class name
 		no_entry => $args{'no_entry'} || 0,
 	}, $class;
 }
 
-# Can also be run as a class level __PACKAGE__::DB::init(directory => '../databases')
-sub init {
-	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
-
-	$directory ||= $args{'directory'};
-	$logger ||= $args{'logger'};
-	$cache ||= $args{'cache'};
-}
-
-sub set_logger {
+sub set_logger
+{
 	my $self = shift;
 
 	my %args;
 
 	if(ref($_[0]) eq 'HASH') {
 		%args = %{$_[0]};
-	} elsif(!ref($_[0])) {
-		Carp::croak('Usage: set_logger(logger => $logger)');
 	} elsif(scalar(@_) % 2 == 0) {
 		%args = @_;
-	} else {
+	} elsif((scalar(@_) == 1) && ref($_[0])) {
 		$args{'logger'} = shift;
 	}
 
-	$self->{'logger'} = $args{'logger'};
-
-	return $self;
+	if(defined($args{'logger'})) {
+		$self->{'logger'} = $args{'logger'};
+		return $self;
+	}
+	Carp::croak('Usage: set_logger(logger => $logger)')
 }
 
 # Open the database.
@@ -245,7 +274,6 @@ sub _open {
 			# Text::CSV::Slurp->import();
 			# $self->{'data'} = Text::CSV::Slurp->load(file => $slurp_file, %options);
 
-			if(0) {
 			require Text::xSV::Slurp;
 			Text::xSV::Slurp->import();
 
@@ -273,7 +301,6 @@ sub _open {
 			foreach my $d(@data) {
 				$self->{'data'}[$i++] = $d;
 			}
-			}
 			$self->{'type'} = 'CSV';
 		} else {
 			$slurp_file = File::Spec->catfile($dir, "$table.xml");
@@ -285,6 +312,7 @@ sub _open {
 				}
 				$dbh->func($table, 'XML', $slurp_file, 'xmlsimple_import');
 			} else {
+				###########
 				my @call_details = caller(0);
 				throw Error::Simple("Can't open $slurp_file called from " .
 					$call_details[2] . ' of ' . $call_details[1]);
@@ -316,8 +344,6 @@ sub selectall_hash {
 	my $table = $self->{table} || ref($self);
 	$table =~ s/.*:://;
 
-	$self->_open() if(!$self->{$table});
-
 	if((scalar(keys %params) == 0) && $self->{'data'}) {
 		if($self->{'logger'}) {
 			$self->{'logger'}->trace("$table: selectall_hash fast track return");
@@ -331,6 +357,8 @@ sub selectall_hash {
 	# if((scalar(keys %params) == 1) && $self->{'data'} && defined($params{'entry'})) {
 	# }
 
+	$self->_open() if(!$self->{$table});
+
 	my $query;
 	my $done_where = 0;
 	if(($self->{'type'} eq 'CSV') && !$self->{no_entry}) {
@@ -339,6 +367,7 @@ sub selectall_hash {
 	} else {
 		$query = "SELECT * FROM $table";
 	}
+
 	my @query_args;
 	foreach my $c1(sort keys(%params)) {	# sort so that the key is always the same
 		my $arg = $params{$c1};
@@ -346,7 +375,7 @@ sub selectall_hash {
 			if($self->{'logger'}) {
 				$self->{'logger'}->fatal("selectall_hash $query: argument is not a string");
 			}
-			throw Error::Simple("$query: argument is not a string");
+			throw Error::Simple("$query: argument is not a string: " . ref($arg));
 		}
 		if(!defined($arg)) {
 			my @call_details = caller(0);
@@ -390,12 +419,20 @@ sub selectall_hash {
 			$key .= ' ' . join(', ', @query_args);
 		}
 		if(my $rc = $c->get($key)) {
+			if($self->{'logger'}) {
+				$self->{'logger'}->debug('cache HIT');
+			}
 			# This use of a temporary variable is to avoid
 			#	"Implicit scalar context for array in return"
 			# return @{$rc};
 			my @rc = @{$rc};
 			return @rc;
 		}
+		if($self->{'logger'}) {
+			$self->{'logger'}->debug('cache MISS');
+		}
+	} elsif($self->{'logger'}) {
+		$self->{'logger'}->debug('cache not used');
 	}
 
 	if(my $sth = $self->{$table}->prepare($query)) {
@@ -409,7 +446,7 @@ sub selectall_hash {
 			push @rc, $href;
 		}
 		if($c && wantarray) {
-			$c->set($key, \@rc, '1 hour');
+			$c->set($key, \@rc, $self->{'cache_duration'});
 		}
 
 		return @rc;
@@ -490,7 +527,7 @@ sub fetchrow_hashref {
 	$sth->execute(@query_args) || throw Error::Simple("$query: @query_args");
 	if($c) {
 		my $rc = $sth->fetchrow_hashref();
-		$c->set($key, $rc, '1 hour');
+		$c->set($key, $rc, $self->{'cache_duration'});
 		return $rc;
 	}
 	return $sth->fetchrow_hashref();
@@ -505,11 +542,9 @@ sub execute {
 
 	if(ref($_[0]) eq 'HASH') {
 		%args = %{$_[0]};
-	} elsif(ref($_[0])) {
-		Carp::croak('Usage: execute(query => $query)');
 	} elsif(scalar(@_) % 2 == 0) {
 		%args = @_;
-	} else {
+	} elsif((scalar(@_) == 1) && !ref($_[0])) {
 		$args{'query'} = shift;
 	}
 
